<?php

namespace ShineSoftware\Constriv\Controller\Hosted;

use Magento\Checkout\Helper\Data as HelperData;
use Magento\Framework\App\Action\Context;
use Magento\Sales\Model\OrderFactory;
use Magento\Quote\Model\QuoteFactory;
use ShineSoftware\Constriv\Model\Logger\Shinelogger;
use Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface;
use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Response extends AbstractHosted {


    /**
     * @var OrderFactory
     */
    protected $_modelOrderFactory;

    /**
     * @var QuoteFactory
     */
    protected $_modelQuoteFactory;

    /**
     * @var HelperData
     */
    protected $_helperData;
    
    /**
     *
     * @var BuilderInterface
     */
    protected $_transactionBuilder;
    
    /**
     * @var OrderSender
     */
    protected $orderSender;
    
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $config;
    
    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $invoiceService;
    
    /**
     * @var \Magento\Framework\DB\TransactionFactory
     */
    protected $transactionFactory;

    public function __construct(
            Context $context, 
            QuoteFactory $modelQuoteFactory, 
            Shinelogger $logLoggerInterface, 
            OrderFactory $modelOrderFactory, 
            HelperData $helperData,
            BuilderInterface $transactionBuilder,
            OrderSender $orderSender,
            ScopeConfigInterface $config,
            \Magento\Framework\DB\TransactionFactory $transactionFactor,
            \Magento\Sales\Model\Service\InvoiceService $invoiceService) {
        $this->_logger = $logLoggerInterface;
        $this->_modelOrderFactory = $modelOrderFactory;
        $this->_helperData = $helperData;
        $this->_modelQuoteFactory = $modelQuoteFactory;
        $this->_transactionBuilder = $transactionBuilder;
        $this->orderSender = $orderSender;
        $this->config = $config;
        $this->invoiceService = $invoiceService;
        $this->transactionFactory = $transactionFactor;

        parent::__construct($context, $modelQuoteFactory, $logLoggerInterface);
    }

    public function execute() {
        $storeManager = $this->_objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        try {
            $paymentid = $this->getRequest()->getParam('paymentid');
            $trackid = $this->getRequest()->getParam('trackid');
            $responsecode = $this->getRequest()->getParam('responsecode');
            $data = $this->getRequest()->getParams();
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - response: " . json_encode($data));
            ob_clean();
            if (empty($trackid)) {
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - TrackID is empty! or you are trying to call the bank response page manually!");
                $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('constriv/hosted/error');
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
                die($redirect);
            }
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - the bank server replies with these parameters: " . json_encode($this->getRequest()->getParams()));
            if (!empty($responsecode) && $responsecode != "00") {
                $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('constriv/hosted/error');
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
                die($redirect);
            }
            $order = $this->_modelOrderFactory->create()->loadByIncrementId($trackid);
            if (!$order) {
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Order has been not found!");
                $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('constriv/hosted/error');
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
                die($redirect);
            }
            if (!empty($data['result']) && ($data['result'] == "AUTHORISED" || $data['result'] == "CAPTURED" || $data['result'] == "APPROVED")) {
                $this->completeOrder($order, $data);
                $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('checkout/onepage/success?paymentid=' . $paymentid, ['_current' => true]);
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
                die($redirect);
            } else {
                if ($this->_cancelPayment()) {
                    $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('constriv/hosted/error', ['_current' => true]);
                } else {
                    $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('checkout/onepage/failure/', ['_current' => true]);
                }
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
                die($redirect);
            }
        } catch (\Exception $e) {
            $this->_logger->debug("Magento error: " . $e->getMessage());
            $redirect = "REDIRECT=" . $storeManager->getStore()->getUrl('constriv/hosted/error');
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect");
            die($redirect);
        }
        die();
    }

    protected function _cancelPayment($errorMsg = '') {
        $errorMsg = trim(strip_tags($errorMsg));
        $this->_logger->debug(__METHOD__ . " - Bank Response: " . $errorMsg);
        $this->_checkoutHelper->cancelCurrentOrder($errorMsg);
        if ($this->_checkoutSession->restoreQuote()) {
            $this->_logger->debug(__METHOD__ . " - Quote has been restored! The user will be redirect to the cart!");
            return true;
        } else {
            $this->_logger->debug(__METHOD__ . " - Quote has been NOT restored! The user will be redirected to the failure page!");
            return false;
        }
    }
    
    protected function completeOrder($order, $paymentData) {
        $this->_createTransaction($order, $paymentData);
        $this->_sendOrderEmail($order);
        $this->_logger->debug(__METHOD__ . " - should create invoice: " . $this->config->getValue('autoinvoice', 'store', $order->getStoreId()));
        if ($this->config->isSetFlag('payment/constriv_hosted/autoinvoice', 'store', $order->getStoreId())) {
            $this->_createInvoice($order);
        }
    }

    /**
     * 
     * @param \Magento\Sales\Model\Order $order
     * @param array $paymentData
     * @return int
     */
    protected function _createTransaction($order = null, $paymentData = array()) {
        try {
            $payment = $order->getPayment();
            $payment->setLastTransId($paymentData['paymentid']);
            $payment->setTransactionId($paymentData['paymentid']);
            $payment->setAdditionalInformation($paymentData);
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                    $order->getGrandTotal()
            );
            $message = __('The authorized amount is %1.', $formatedPrice);
            $trans = $this->_transactionBuilder;
            if ($this->config->getValue('payment/constriv_hosted/request_type', 'store', $order->getStoreId()) == 'authorize_capture') {
                $type = \Magento\Sales\Api\Data\TransactionInterface::TYPE_ORDER;
                $isClosed = true;
            } else {
                $type = \Magento\Sales\Api\Data\TransactionInterface::TYPE_AUTH;
                $isClosed = false;  //this makes the capture online possible
            }
            $transaction = $trans->setPayment($payment)
                    ->setOrder($order)
                    ->setTransactionId($paymentData['paymentid'])
                    ->setAdditionalInformation($paymentData)
                    ->setFailSafe(true)
                    ->build($type)
                    ->setIsClosed($isClosed);
            $payment->addTransactionCommentsToOrder(
                    $transaction, $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->setState("processing")->setStatus($this->config->getValue('payment/constriv_hosted/authorized_order_status', 'store', $order->getStoreId()));
            $order->setTotalPaid($order->getGrandTotal());
            $order->addStatusHistoryComment('Transaction confirmed');
            $order->save();
            return $transaction->save()->getTransactionId();
        } catch (\Exception $e) {
            $this->_logger->debug(__METHOD__ . " - Failed to create transaction: " . $e->getMessage());
        }
    }
    
    
    protected function _sendOrderEmail($order) {
        if ($order->getCanSendNewEmailFlag()) {
            try {
                $this->orderSender->send($order);
            } catch (\Exception $e) {
                $this->_logger->critical($e);
            }
        }
    }
    
    protected function _createInvoice($order) {
//        $this->_logger->debug(__METHOD__ . " - Can invoice? " . var_export($order->canInvoice(), TRUE));
        if ($order->canInvoice()) {
            $this->_logger->debug(__METHOD__ . " - Trying to create invoice");
            $invoice = $this->invoiceService->prepareInvoice($order);
            if ($this->config->getValue('payment/constriv_hosted/request_type', 'store', $order->getStoreId()) == 'authorize_capture') {
                $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_OFFLINE);
            } else {
                $invoice->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::CAPTURE_ONLINE);
            }
            $invoice->register();
            $transactionSave = $this->transactionFactory->create()
                    ->addObject($invoice)
                    ->addObject($invoice->getOrder());
            $transactionSave->save();
        } else {
            $this->_logger->debug(__METHOD__ . " - Cannot create invoice");
        }
    }
}
