<?php

namespace ShineSoftware\Constriv\Gateway\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use ShineSoftware\Constriv\Model\Source\CurrenciesFactory;

class VoidRequest implements BuilderInterface {

    private $config;
    protected $_sourceCurrenciesFactory;
    public $_logger;

    public function __construct(
    ConfigInterface $config,
    CurrenciesFactory $sourceCurrenciesFactory,
    \ShineSoftware\Constriv\Model\Logger\Shinelogger $logger
    ) {
        $this->config = $config;
        $this->_sourceCurrenciesFactory = $sourceCurrenciesFactory;
        $this->_logger = $logger; 
    }

    public function build(array $buildSubject) {
        if (!isset($buildSubject['payment']) || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }
        $paymentDO = $buildSubject['payment'];
        $order = $paymentDO->getOrder();
        $currencies = $this->_sourceCurrenciesFactory->create();
        $payment = $paymentDO->getPayment();
        if (!$payment instanceof OrderPaymentInterface) {
            throw new \LogicException('Order payment should be provided.');
        }
        $addInfo = $payment->getAdditionalInformation();
        if (empty($addInfo)) {
            throw new \Exception('No Authorization transaction information have been found!');
        }
        $data = [
            'id' => $this->config->getValue('id', $order->getStoreId()),
            'password' => $this->config->getValue('password', $order->getStoreId()),
            'action' => 9,
            'currencycode' => $currencies->findCurrencyIsoCode($payment->getOrder()->getOrderCurrencyCode()),
            'amt' => number_format($order->getGrandTotalAmount(), 2, '.', ''),
            'paymentid' => $addInfo['paymentid'],
            'transid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
            'url' => $this->getEndpointUrl('callback', $order->getStoreId()),
        ];
        return $data;
    }
    
    
    private function getEndpointUrl($type = 'payment_init', $storeId = 1) {
        if ($this->config->getValue("debug", $storeId)) {
            return $this->config->getValue("uri/test/$type", $storeId);
        } else {
            return $this->config->getValue("uri/live/$type", $storeId);
        }
    }

}

;
